@@ -327,8 +327,10 @@
 #' 
 #' @import data.table
 #' @import Epi
+#' @family splitting_related
+#' @family aggregation_related
 #' @seealso
-#' \code{\link{splitMulti}}, \code{\link[Epi]{Lexis}}, \code{\link{survtab}}, \code{\link{relpois}}, \code{\link{popmort}} \code{\link{sir}}
+#' \code{\link[Epi]{Lexis}}, \code{\link{popmort}}
 #' @export
 lexpand <- function(data, 
                     birth=NULL, entry=NULL, exit=NULL, event=NULL,
@@ -901,7 +903,7 @@ lexpand <- function(data,
                                        ", paste0(l[[1]]))
     if (verbose) cat("Aggregation done. \n")
     
-    if (!getOption("popEpi.datatable") && is.data.table(l)) setDFpe(l)
+    if (!return_DT() && is.data.table(l)) setDFpe(l)
     
   } else {
     
@@ -916,7 +918,7 @@ lexpand <- function(data,
     setattr(l, "time.scales", c("fot","per","age"))
     setattr(l, "breaks", breaks)
     setattr(l, "class", c("Lexis","data.table","data.frame"))
-    if (!getOption("popEpi.datatable") && is.data.table(l)) setDFpe(l)
+    if (!return_DT() && is.data.table(l)) setDFpe(l)
     
     
     
