@@ -37,6 +37,7 @@
 #' E.g. \code{"Missing \%\%NA_COUNTS\%\% observations due to derpness."}
 #' @examples 
 #' library(survival)
+#' library(data.table)
 #' 
 #' makeWeightsDT <- popEpi:::makeWeightsDT ## this avoids errors during tests
 #' 
@@ -203,6 +204,15 @@ makeWeightsDT <- function(data, values = NULL,
     stop("Variables to adjust by were defined but no weights were supplied.")
   }
   
+  if (!length(adVars)) {
+    
+    if (!is.null(weights)) {
+      message("NOTE: Weights ignored since no adjusting variables given")
+    }
+    
+    weights <- NULL
+  }
+  
   # variables to sum -----------------------------------------------------------
   if (!is.list(values)) stop("Argument 'values' must be a list ",
                              "(internal error: complain to the package",
@@ -394,16 +404,14 @@ makeWeightsDT <- function(data, values = NULL,
                             "to the pkg maintainer if you see this."
                           ))
         
-        weights <- mapply(function(levs, colname) {
-          setkeyv(data, colname)
-          data[.(levs), lapply(.SD, sum), .SDcols = eval(iwVar), by = .EACHI]
-        }, 
-        colname = names(adjust), levs = adjust,
-        SIMPLIFY = FALSE)
-        
-        weights <- lapply(weights, function(x) {
-          x[[iwVar]]
+        weights <- lapply(seq_along(adjust), function(i) {
+          cn <- names(adjust)[i]
+          le <- unique(adjust[[i]])
+          le <- structure(list(le), names = cn)
+          data[le, lapply(.SD, sum), .SDcols = iwVar, by = .EACHI, on = cn][[iwVar]]
         })
+        names(weights) <- names(adjust)
+        
         
         setcolsnull(data, iwVar)
         setkeyv(data, c(prVars, adVars, boVars))
